/*  Elm2Waf.C - A program should a user ever want to (but I can't fathom
    why...) revert to the good ol' way Waffle does mail, or is bringing old
    mailboxes from a Unix system onto a Waffle site.
    Copyright (C) 1992 Ian Justman, 28 June 1992.  Release 1.0
    Compiled with Turbo C 2.0                                               */

#include <stdio.h>
#include <errno.h>
#include <fcntl.h>

#define SOH 1
#define TRUE -1
#define FALSE 0

main(argc, argv)
int argc;
char *argv[];
{
    char *in_file, *out_file;
    int ch, soh_cnt = 0, i, soh_flag=FALSE, cat_flag=TRUE, test;
    FILE *in, *out;

    printf("Elm2Waf 1.0 - Convert Elm mailbox files for use with Waffle 1.65");
    printf(".\nCopyright (C) 1992 by Ian R. Justman (ianj@ijpc.UUCP)");
    printf("\n\n");

    /* The program first tries to determine if the proper number of arguments
     * have been entered.  Give the guy some help if he hasn't.
     */

    if((argc <= 2) || (argc >= 4)) {
        fprintf(stderr, "Syntax:  elm2waf inpath outpath\n\n");
        exit(1);
    }

    /* Attempt to open the input file which must exist.  If it does not, or
     * something else goes hideously wrong, exit.
     */

    in_file = argv[1];
    out_file = argv[2];
    if((in=fopen(in_file, "rb")) == (FILE *) NULL) {
        fprintf(stderr, "Problems trying to open input file.\n\n");
        exit(1);
    }

    /* Test to see if the output file exists before processing.  If it does,
     * clear the concatenation flag (cat_flag) and process input into
     * a new file.  Otherwise, there may be too many SOHes for Waffle
     * to handle in the middle of the file and there's no telling how Waffle
     * would handle this.
     */

    if((test = open(out_file, O_RDONLY, 0666)) == -1) cat_flag=FALSE;
    close(test);

    /* Attempt to open the output file which need not exist, but if one does,
     * output will be concatenated to it.  If it does not, one will be
     * created.  If someting goes nuts, exit and tell the poor soul
     * wha' happened.
     */

    if((out=fopen(out_file, "a+")) == (FILE *) NULL) {
        fprintf(stderr, "Problems trying to open output file.\n\n");
        exit(1);
    }

    /* Here's the real engine of the program.  Of course, it watches for
     * the end-of-file condition, but it also watches for other critical
     * characters as well, like the presense of an SOH character, which is
     * not immediately output, but is counted, as well as any others in the
     * sequence it may find.  It also looks for the first non-SOH
     * character after the last SOH.  When one is encountered, a number of
     * SOHes, definded in soh_count, is put to the stream "out".  If the
     * character encountered is a carriage return, it is not output in
     * keeping with Waffle 1.65's mailbox format.  Instead, the next charac-
     * ter is fetched and is processed.  After the EOF condition is en-
     * countered, a final sequence of 4 SOHes are output to the "out" stream
     * and both the input and output files are closed.
     *
     * If the output file specified on the command line actually exists,
     * a concatenation flag, which is already set, tells the SOH routine
     * not to output a sequence of SOHes, even though they are counted any-
     * way, and then clears the flag when the first non-SOH character is
     * encountered.  On the other hand, if the file does not exist, the con-
     * catenation flag (cat_flag) is cleared and a new file is created.
     */

    while ((ch = getc (in)) != EOF) {
        if(ch == SOH) {
            soh_cnt++;
            soh_flag=TRUE;
        }

        if((ch != SOH) && (soh_cnt > 0)) {
            if(cat_flag == FALSE) for(;soh_cnt > 0; putc(SOH, out), soh_cnt--);
            else {
                soh_cnt=0;
                cat_flag=FALSE;
            }
            if(ch != '\n') putc(ch, out);
        }
    if(((ch != SOH) && ((soh_flag == TRUE) && (ch != '\n'))) || (soh_flag == FALSE)) {
            putc(ch, out);
            soh_flag=FALSE;
	}
    }
    for (i=4; i > 0; i--, putc(SOH, out));
    if ((fclose(in) == EOF) || (fclose(out) == EOF)) {
        fprintf(stderr, "Problems trying to close files.\n\n");
    }
    printf("Successful conversion of %s to %s.\n\n", in_file,
        out_file);
}
