/*  Waf2Elm.C - A program to convert existing Waffle 1.65-and-later-style
    MAILBOX files for use with PCElm (tested with PCElm 3.0 and PCElm 3.01).
    Copyright (C) 1992 Ian Justman, 28 June 1992.  Release 1.0
    Compiled with Turbo C 2.0                                               */

#include <stdio.h>

#define SOH 1

main(argc, argv)
int argc;
char *argv[];
{
    char *in_file, *out_file;
    int ch, soh_cnt = 0;
    FILE *in, *out;

    /* The program first tries to determine if the proper number of arguments
     * have been entered.  Give the guy some help if he hasn't.
     */

    printf("Waf2Elm 1.0 - Convert Waffle 1.65 mailbox files for use with Elm");
    printf(" or PCElm.\nCopyright (C) 1992 by Ian R. Justman (ianj@ijpc.UUCP)");
    printf("\n\n");
    if((argc <= 2) || (argc >= 4)) {
        fprintf(stderr, "Syntax:  waf2elm inpath outpath\n\n");
        exit(1);
    }

    /* Attempt to open the input file which must exist.  If it does not exist,
     * or a mushroom cloud goes up inside the drive, exit, and inform the
     * guy.
     */

    in_file = argv[1];
    out_file = argv[2];
    if((in=fopen(in_file, "r")) == (FILE *) 0) {
        fprintf(stderr, "Problems trying to open input file.\n\n");
        exit(1);
    }
    if((out=fopen(out_file, "a+b")) == (FILE *)0) {
        fprintf(stderr, "Problems trying to open output file.\n\n");
        exit(1);
    }

    /* Here's the real engine of the program.  Of course, it watches for
     * the end-of-file condition, but it also watches for other critical
     * characters as well, like the presense of an SOH character, which is
     * not immediately output, but is counted, as well as any others in the
     * sequence it may find.  It also looks for the first non-SOH
     * character after the last SOH.  When one is encountered, a number of
     * SOHes, definded in soh_count, is put to the stream "out".  The the same
     * character which triggers the SOHes to be dumped, a newline is sent.  If
     * not, Elm will not be able to parse the messages at all.  After the new-
     * line is sent, the character is sent.
     *
     * If the character encountered is a carriage return, though it is unlikely,
     * it will be output.
     *
     * When an EOF condition is encountered, whether or not SOHes have been
     * counted, both the input and output files are closed.  Most likely the EOF
     * will come after an SOH header.
     *
     * Since there is no SOH at the end of the mail file for Elm, should one
     * exist, no special processing is necessary and the output is append
     * directly to it by opening the file in append binary update mode (ab+).
     * It is opened in binary mode because it is desirable to use LF-delimited
     * lines, although CR+LF-delimited lines work.  The original file is opened
     * in text mode, so it doesn't really matter what the original file has.
     *
     * If the output file doesn't exist, it is created.
     */

    while ((ch = getc (in)) != EOF) {
        if(ch == SOH) soh_cnt++;
        if((ch != SOH) && (soh_cnt > 0) && (ch != EOF)) {
            for(;soh_cnt > 0; putc(SOH, out), soh_cnt--);
            putc('\n', out);
            putc(ch, out);
        }
        else if((ch != SOH) && (ch != '\r')) putc(ch, out);
        else if(ch == '\n') putc('\n', out);
    }
    if ((fclose(in) == EOF) || (fclose(out) == EOF)) {
        fprintf(stderr, "Problems trying to close files.\n\n");
        exit(1);
    }
    printf("Successful conversion of %s to %s.\n\n", in_file, out_file);
}

